package ovh.herisson.Clyde.EndPoints;

import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.Repositories.TokenRepository;
import ovh.herisson.Clyde.Repositories.UserRepository;
import ovh.herisson.Clyde.Services.*;
import ovh.herisson.Clyde.Tables.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")

public class MockController {
    private final BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    public final UserRepository userRepo;
    public final TokenRepository tokenRepo;
    public final TokenService tokenService;
    public final CurriculumCourseService CurriculumCourseService;
    public final CurriculumService curriculumService;
    public final CourseService courseService;

    public final InscriptionService inscriptionService;

    public final LessonService lessonService;
    public final ScheduleService scheduleService;
    public final ScheduleLessonService scheduleLessonService;

    ArrayList<User> mockUsers;


    public MockController(UserRepository userRepo, TokenRepository tokenRepo, TokenService tokenService, CurriculumCourseService CurriculumCourseService, CurriculumService curriculumService, CourseService courseService, InscriptionService inscriptionService, LessonService lessonService, ScheduleService scheduleService, ScheduleLessonService scheduleLessonService){
        this.tokenRepo = tokenRepo;
        this.userRepo = userRepo;
        this.tokenService = tokenService;
        this.CurriculumCourseService = CurriculumCourseService;
        this.curriculumService = curriculumService;
        this.courseService = courseService;
        this.inscriptionService = inscriptionService;
        this.lessonService = lessonService;
        this.scheduleService = scheduleService;
        this.scheduleLessonService = scheduleLessonService;
    }

    /** Saves an example of each user type by :
     * email : FooRole@FooRole.com, password : FooRole and token : FooRole
     * For example the admin as "admin@admin.com" as email and "admin" as both password and token
     * They all have silly names
     */

    @PostMapping("/mock")
    public void postMock(){

        // user part


        User herobrine = new User("brine","hero","admin@admin.com","behind","ShadowsLand",new Date(0), null,Role.Admin,passwordEncoder.encode("admin"));
        User joe = new User("Mama","Joe","student@student.com","roundabout","England",new Date(0), null,Role.Student,passwordEncoder.encode("student"));
        User meh = new User("Polo","Marco","secretary@secretary.com","a Box","Monaco",new Date(0), null,Role.Secretary,passwordEncoder.encode("secretary"));
        User joke = new User("Gaillard","Corentin","teacher@teacher.com","lab","faculty",new Date(0), null,Role.Teacher,passwordEncoder.encode("teacher"));
        User jojo = new User("Bridoux","Justin","teacher2@teacher2.com","lab","faculty",new Date(0), null,Role.Teacher,passwordEncoder.encode("teacher"));
        User lena = new User("Louille","Lena","inscriptionService@InscriptionService.com","no","yes",new Date(0), null,Role.InscriptionService,passwordEncoder.encode("inscriptionService"));
        mockUsers = new ArrayList<>(Arrays.asList(herobrine,joe,meh,joke,lena,jojo));

        userRepo.saveAll(mockUsers);

        // Course / Curriculum part

        Curriculum infoBab1 = new Curriculum(1,"info");
        Curriculum chemistryBab1 = new Curriculum(1,"chemistry");
        Curriculum psychologyBab1 = new Curriculum(1,"psychology");

        curriculumService.save(infoBab1);
        curriculumService.save(chemistryBab1);
        curriculumService.save(psychologyBab1);


        Course progra1 = new Course(5,"Programmation et algorithmique 1",joke);
        Course chemistry1 = new Course(12, "Thermochimie",joke);
        Course psycho1 = new Course(21, "Neuroreaction of isolated brain cells",joke);
        Course commun = new Course(2, "cours commun",joke);

        courseService.save(progra1);
        courseService.save(chemistry1);
        courseService.save(psycho1);
        courseService.save(commun);


        CurriculumCourseService.save(new CurriculumCourse(infoBab1,progra1));
        CurriculumCourseService.save(new CurriculumCourse(infoBab1,commun));

        CurriculumCourseService.save(new CurriculumCourse(psychologyBab1,psycho1));
        CurriculumCourseService.save(new CurriculumCourse(psychologyBab1,commun));


        CurriculumCourseService.save(new CurriculumCourse(chemistryBab1,commun));
        CurriculumCourseService.save(new CurriculumCourse(chemistryBab1,chemistry1));


        InscriptionRequest inscriptionRequest = new InscriptionRequest("helen","prenom","non","helen@gmail.com","america",new Date(),(long) 1,RequestState.Pending,"yes.png","password");

        inscriptionService.save(inscriptionRequest);


        //Schedule part

        Lesson lesson_0_progra1 = new Lesson(progra1, "Mon Apr 01 2024 08:15", "Mon Apr 01 2024 10:15","rgb(0,50,100)","A0B2");
        Lesson lesson_0_chemistry1 = new Lesson(chemistry1, "Wed Mar 27 2024 08:15", "Wed Mar 27 2024 09:15","rgb(100,50,0)","A0B2");
        Lesson lesson_0_psycho1 = new Lesson(psycho1, "Sun Mar 24 2024 10:30 ","Sun Mar 24 2024 12:30 ","rgb(100,50,100)", "A0B2");
        Lesson lesson_1_progra1 = new Lesson(progra1, "Mon Apr 02 2024 13:30", "Mon Apr 02 2024 15:30","rgb(0,50,100)","A0B2");
        Lesson lesson_0_commun = new Lesson(commun, "Mon Apr 01 2024 10:30", "Mon Apr 01 2024 12:30","rgb(0,50,100)","A0B2");

        Schedule infoBab1Schedule = new Schedule(infoBab1);
        Schedule chemistryBab1Schedule = new Schedule(chemistryBab1);
        Schedule psychoBab1Schedule = new Schedule(psychologyBab1);

        scheduleService.save(infoBab1Schedule);
        scheduleService.save(chemistryBab1Schedule);
        scheduleService.save(psychoBab1Schedule);

        lessonService.save(lesson_0_progra1);
        lessonService.save(lesson_0_chemistry1);
        lessonService.save(lesson_0_commun);
        lessonService.save(lesson_0_psycho1);
        lessonService.save(lesson_1_progra1);

        scheduleLessonService.save(new ScheduleLesson(infoBab1Schedule,lesson_0_progra1));
        scheduleLessonService.save(new ScheduleLesson(infoBab1Schedule,lesson_1_progra1));
        scheduleLessonService.save(new ScheduleLesson(infoBab1Schedule,lesson_0_commun));

        scheduleLessonService.save(new ScheduleLesson(chemistryBab1Schedule,lesson_0_chemistry1));
        scheduleLessonService.save(new ScheduleLesson(chemistryBab1Schedule,lesson_0_commun));

        scheduleLessonService.save(new ScheduleLesson(psychoBab1Schedule,lesson_0_psycho1));
        scheduleLessonService.save(new ScheduleLesson(psychoBab1Schedule,lesson_0_commun));

    }
}

