package ovh.herisson.Clyde.Services;

import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Repositories.LessonRepository;
import ovh.herisson.Clyde.Repositories.ScheduleLessonRepository;
import ovh.herisson.Clyde.Repositories.ScheduleRepository;
import ovh.herisson.Clyde.Tables.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

@Service
public class ScheduleLessonService {

    private final ScheduleLessonRepository scheduleLessonRepo;

    private final LessonRepository lessonRepo;
    private final ScheduleRepository scheduleRepo;
    public ScheduleLessonService(ScheduleLessonRepository scheduleLessonRepo, LessonRepository lessonRepo, ScheduleRepository scheduleRepo) {
        this.scheduleLessonRepo = scheduleLessonRepo;
        this.lessonRepo = lessonRepo;
        this.scheduleRepo = scheduleRepo;
    }

    public void save(ScheduleLesson scheduleLesson){
        scheduleLessonRepo.save(scheduleLesson);
    }

    public Schedule getScheduleByCurriculum(Curriculum curriculum){
        return scheduleLessonRepo.findScheduleByCurriculum(curriculum);
    }

    public Map<String , Object> getDepthScheduleByCurriculum(Curriculum curriculum){
        if(curriculum == null)
            return null;

        HashMap<String , Object> toReturn = new HashMap<>();
        ArrayList<Map<String, Object>> lessons = new ArrayList<>();
        Iterable<Lesson> foundLessons = scheduleLessonRepo.findLessonByCurriculum(curriculum);

        for (Lesson l: foundLessons){
            lessons.add(ProtectionService.lessonWithoutPassword(l));
        }
        toReturn.put("lessons",lessons);
        toReturn.put("scheduleId" , scheduleLessonRepo.findScheduleByCurriculum(curriculum).getScheduleID());
        toReturn.put("curriculum", curriculum);
        return toReturn;
    }

    public Map<String , Object> getDepthScheduleBySchedule(Schedule schedule){
        if(schedule == null)
            return null;

        HashMap<String , Object> toReturn = new HashMap<>();
        ArrayList<Map<String, Object>> lessons = new ArrayList<>();
        Iterable<Lesson> foundLessons = scheduleLessonRepo.findLessonByCurriculum(schedule.getCurriculum());

        for (Lesson l: foundLessons){
            lessons.add(ProtectionService.lessonWithoutPassword(l));
        }
        toReturn.put("lessons",lessons);
        toReturn.put("scheduleId" , schedule.getScheduleID());
        toReturn.put("curriculum", schedule.getCurriculum());
        return toReturn;
    }

    public Iterable<Map<String,Object>> getAllSchedule(){
        ArrayList<Map<String,Object>> toReturn = new ArrayList<>();

        for (Schedule schedule: scheduleRepo.findAll()){
            toReturn.add(getDepthScheduleBySchedule(schedule));
        }
        return toReturn;
    }

}
